//////////////////////////////////////////////////////////////////////////////
//
// Copyright  2002-2003, Microsoft Corporation.  All rights reserved.
//
// File: Waitmsg.js
//
// Description:
//
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////
// Class: CViewerMessages
//
// Purpose:
//	The CViewerMessages class implements a message queue which can
//	be used to display contextual messages at runtime.  Only messages
//	that remain in the queue for more than two seconds are displayed.
//	If multiple messages have remained in the queue for more than two
//	seconds, the message that has been queued for the least amount of
//	time will be displayed.  This enables long subprocesses to display
//	their own message within the context of a larger operation that has
//	also queued a message.
//
// Parameters:
//	String sViewerPath [in] - Viewer folder path
//
//////////////////////////////////////////////////////////////////////////
function CViewerMessages(sViewerPath)
{
	if (!sViewerPath || typeof(sViewerPath) != "string") return null;

	// Add array methods pop and shift, if IE's jscript version does not support them
	if (!Array.prototype.pop) Array.prototype.pop = ArrayPop;
	if (!Array.prototype.shift) Array.prototype.shift = ArrayShift;
	
	////////////////////////////////////////////////////////
	//PRIVATE VARIABLES
	////////////////////////////////////////////////////////

	var aMsg = new Array();			// Array to hold queued messages
    	var hMsgWin = null;			// Message window handle
	var sLastID = "";			// Last message identifier
	var oCurrMsg = null;			// Current message object
	var sPath = AppendSlash(sViewerPath);	// Viewer folder path

	////////////////////////////////////////////////////////
	//PUBLIC METHODS.
	////////////////////////////////////////////////////////
	
	//--------------------------------------------------------------------
	// Method: AddMessage
	//
 	// Purpose: Adds a new message to the array.
	//
 	// Parameters:
	//	String sMsg [in] - The string to add
 	//
	// Returns: String - A token used to identify the message
	//--------------------------------------------------------------------
	function CViewerMessages.prototype.AddMessage(sMsg)
	{
		var sID = "";

		try
		{
			if (sMsg && "string" == typeof(sMsg))
			{
				var oNewMsg = new Object();
				oNewMsg.dateTime = new Date();
	   			oNewMsg.sMsg = sMsg;
				oNewMsg.sID = window.document.uniqueID;
				oNewMsg.nPopTok = 0;
		
				aMsg[aMsg.length] = oNewMsg;

				DisplayMessage();

				sID = oNewMsg.sID;
			}
		}
		catch(e)
		{
		}

		return sID;
	}

	//--------------------------------------------------------------------
	// Method: RemoveMessage
	//
 	// Purpose:
  	//	Removes the specified message from the array.
	//
 	// Parameters:
	//	String sID [in] - Identifier of the message to remove
 	//
	// Returns: None
	//--------------------------------------------------------------------
	function CViewerMessages.prototype.RemoveMessage(sID)
	{
		try
		{
			if (sID && "string" == typeof(sID))
			{
				for (var i = 0; i < aMsg.length; i++)
				{
					if (aMsg[i].sID == sID)	
					{
						if (i == 0)
							aMsg.shift();
						else if (i == aMsg.length)
							aMsg.pop();
						else
						{
							aMsg[i].nPopTok = 1;
							SortBy("nPopTok");
							aMsg.pop();
						}
						break;
					}
				}
			}
		}
		catch(e)
		{
		}
	}

	//--------------------------------------------------------------------
	// Method: FlushMessages
	//
 	// Purpose: Removes all queued messages from the array and closes the
  	//	    message window, if displayed.
	//
 	// Parameters: None
 	//
	// Returns: None
	//--------------------------------------------------------------------
	function CViewerMessages.prototype.FlushMessages()
	{
		try
		{
			aMsg = null;
			sLastID = "";
			hMsgWin.close();
		}
		catch(e)
		{
		}
	}

	////////////////////////////////////////////////////////
	//PRIVATE METHODS.
	////////////////////////////////////////////////////////

	//--------------------------------------------------------------------
	// Method: GetCurrentMessage
	//
 	// Purpose: Gets a message from the array.  A message must be in the array
  	//	    for more than 2 seconds to be fetched.  If multiple messages
	//	    have been queued for greater than 2 seconds, the message that
	//	    has been queued for the least amount of time is fetched.
 	//
 	// Parameters: None
 	//
	// Returns: Object - The current message object
	//--------------------------------------------------------------------
	function GetCurrentMessage()
	{
		var oMsg = null;

		try
		{
			var nPos = -1;
			var nDelta = -1;
			var nCurrDelta = -1;
		
			SortBy("dateTime");

			for (var i = 0; i < aMsg.length; i++)
			{
				nCurrDelta = new Date() - aMsg[i].dateTime;
				if (Math.round((nCurrDelta)/1000) >= 2)
				{
					if (nCurrDelta < nDelta  || nDelta == -1)
					{ 
						nDelta = nCurrDelta;
						nPos = i;
					}
				}
				else
					break;
			}

			oMsg = aMsg[nPos];
		}
		catch(e)
		{
		}

		return oMsg;	
	}

	//--------------------------------------------------------------------
	// Method: DisplayMessage
	//
 	// Purpose:
  	//	Creates/updates a message window and displays a message
	//	that has been in the message array for more than two seconds.
	//	If multiple messages have been in the array for more than two
	//	seconds, it displays the message that has been in the array
	//	for the least amount of time.
	//
 	// Parameters: None
 	//
	// Returns: None
	//--------------------------------------------------------------------
	function DisplayMessage()
	{
		try
		{
			oCurrMsg = GetCurrentMessage();
				
			if ((!hMsgWin || hMsgWin.closed) && oCurrMsg)
			{
				var xOffset = (screen.width - 300)/2, yOffset = (screen.height - 30)/2;
   				var sWinProps = "toolbar=no,location=no,directories=no,menubar=no,scrollbars=no,resizable=no,status=no,height=30,width=300,left=" + xOffset + ",top=" + yOffset;
				hMsgWin = window.open(sPath + "viewer/waitmsg.htm", "", sWinProps);
				if (window.attachEvent) 
				{
					window.attachEvent("onfocus",StealFocus);
					window.attachEvent("onunload",CloseMsgWindow);
				}	
				if (hMsgWin) UpdateMessage();
			}

			else if (hMsgWin && !hMsgWin.closed && oCurrMsg && sLastID != oCurrMsg.sID)
				UpdateMessage();

			else if (hMsgWin && !hMsgWin.closed && !oCurrMsg)
			{
				hMsgWin.close();
				hMsgWin = null;
				if (window.detachEvent) 
				{
					window.detachEvent("onfocus",StealFocus);
					window.detachEvent("onunload",CloseMsgWindow);
				}
			}

			if (aMsg.length) window.setTimeout(DisplayMessage,100);
		}
		catch(e)
		{
		}
	}

	//--------------------------------------------------------------------
	// Method: UpdateMessage
	//
 	// Purpose: Updates the message window with the current message.
  	//
 	// Parameters: None
 	//
	// Returns: None
	//--------------------------------------------------------------------			
	function UpdateMessage()
	{
		if (hMsgWin && oCurrMsg)
		{
			try
			{
				hMsgWin.document.all.idMsg.innerText = oCurrMsg.sMsg;
				hMsgWin.resizeWindow();
				hMsgWin.focus();
				sLastID = oCurrMsg.sID;
			}
			catch(e)
			{
				window.setTimeout(UpdateMessage,5);
			}
		}
	}

	//--------------------------------------------------------------------
	// Method: StealFocus
	//
 	// Purpose: Gives focus to the message window, if one exists.
	//
 	// Parameters: None
 	//
	// Returns: None
	//--------------------------------------------------------------------
	function StealFocus()
	{
		try {hMsgWin.focus();} catch(e) {}
	}

	//--------------------------------------------------------------------
	// Function: CloseMsgWindow
	//
	// Purpose: Closes the message window when the viewer window is closed.
	//
	// Parameters: None
	//
	// Returns: None
	//--------------------------------------------------------------------
	function CloseMsgWindow()
	{
		try {hMsgWin.close();} catch(e) {}
	}

	//--------------------------------------------------------------------
	// Method: SortBy
	//
 	// Purpose:
  	//
	//
 	//
 	// Parameters:
	//	String sProp [in] - 
 	//
	// Returns: None
	//--------------------------------------------------------------------
	function SortBy(sProp)
	{
		sortProp = sProp;
		aMsg.sort(SortFunc);
	}

	//--------------------------------------------------------------------
	// Method: SortFunc
	//
 	// Purpose:
  	//
	//
 	//
 	// Parameters:
	//	Object obj1 [in] - First object to compare
	//	Object obj2 [in] - Second object to compare
 	//
	// Returns: -1, 0 or 1
	//--------------------------------------------------------------------
	function SortFunc(obj1,obj2)
	{
		if (obj1[sortProp] < obj2[sortProp]) return -1;
		else if (obj1[sortProp] > obj2[sortProp]) return 1;
		else return 0;
	}
}